<?php

include('./bdd.php');

class CategorieMetier
{
    /**
     * gestion statique des accès SGBD
     * @var PDO
     */ 
    private static $_pdo;

    /**
     * gestion statique de la requête préparée de selection
     * @var PDOStatement
     */ 
    private static $_pdos_select;

    /**
     * gestion statique de la requête préparée de mise à jour
     *  @var PDOStatement
     */ 
    private static $_pdos_update;

    /**
     * gestion statique de la requête préparée de d'insertion
     * @var PDOStatement
     */ 
    private static $_pdos_insert;

    /**
     * gestion statique de la requête préparée de suppression
     * @var PDOStatement
     */ 
    private static $_pdos_delete;

    /**
     * PreparedStatement associé à un SELECT, calcule le nombre de catégorie de la table
     * @var PDOStatement;
        */
    private static $_pdos_count;

    /**
     * PreparedStatement associé à un SELECT, récupère toutes les catégories
     * @var PDOStatement;
        */
     private static $_pdos_selectAll;



    /**
     * Initialisation de la connexion et mémorisation de l'instance PDO dans Categorie::$_pdo
     */ 
    public static function initPDO()
    {
        self::$_pdo = new PDO("mysql:host=".$_ENV['host'].";dbname=".$_ENV['db'],$_ENV['user'],$_ENV['passwd']);
        // pour récupérer aussi les exceptions provenant de PDOStatement
        self::$_pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    }

    /**
     * préparation de la requête SELECT * FROM Categorie
     * instantiation de self::$_pdos_selectAll
     */
    public static function initPDOS_selectAll()
    {
        self::$_pdos_selectAll = self::$_pdo->prepare('SELECT * FROM Categorie');
    }

    /**
     * méthode statique instanciant Categorie::$_pdo_select
     */ 
    public static function initPDOS_select() {
        self::$_pdos_select = self::$_pdo->prepare('SELECT * FROM Categorie WHERE id_categorie=:idB');
    }

    /**
     * méthode statique instanciant Categorie::$_pdo_update
     */ 
    public static function initPDOS_update() {
        self::$_pdos_update =  self::$_pdo->prepare('UPDATE Categorie SET robe_bouteille=:robeB, sucrage_bouteille=:sucrageB, type_bouteille=:typeB WHERE id_categorie=:idB');
    }

    /**
     * méthode statique instanciant Categorie::$_pdo_insert
     */ 
    public static function initPDOS_insert() {
        self::$_pdos_insert = self::$_pdo->prepare('INSERT INTO Categorie(robe_bouteille, sucrage_bouteille, type_bouteille) VALUES(:robeB,:sucrageB,:typeB)');
    }

    /**
     * méthode statique instanciant Categorie::$_pdo_delete
     */ 
    public static function initPDOS_delete() {
        self::$_pdos_delete = self::$_pdo->prepare('DELETE FROM Categorie WHERE id_categorie=:idB');
    }

    /**
     * préparation de la requête SELECT COUNT(*) FROM Categorie
     * instantiation de self::$_pdos_count
     */
    public static function initPDOS_count() {
        if (!isset(self::$_pdo))
            self::initPDO();
        self::$_pdos_count = self::$_pdo->prepare('SELECT COUNT(*) FROM Categorie');
    }



    /**
     * numéro de la catégorie (identifiant dans la table Categorie)
     * @var int
     */ 
    protected $id_categorie;

    /**
     * robe de la bouteille
     * @var string
     */ 
    protected $robe_bouteille;

    /**
     * sucrage de la bouteille
     *   @var string
     */ 
    protected $sucrage_bouteille;

    /**
     * type de la bouteille
     *   @var string
     */ 
    protected $type_bouteille;

    /**
     * attribut interne pour différencier les nouveaux objets des objets créés côté applicatif de ceux issus du SGBD
     * @var bool
     */ 
    private $nouveau = TRUE;


    /**
     * @return $this->id_categorie
     */ 
    public function getId_categorie() : int {
        return $this->id_categorie;
    }

    /**
     * @param $id_categorie
     */ 
    public function setId_categorie($id_categorie): void {
        $this->id_categorie = $id_categorie;
    }

    /**
     * @return $this->robe_bouteille
     */ 
    public function getRobe_bouteille() : string {
        return $this->robe_bouteille;
    }

    /**
     * @param $robe_bouteille
     */ 
    public function setRobe_bouteille($robe_bouteille): void {
        $this->robe_bouteille = $robe_bouteille;
    }

    /**
     * @return $this->sucrage_bouteille
     */ 
    public function getSucrage_bouteille() : string {
        return $this->sucrage_bouteille;
    }

    /**
     * @param $sucrage_bouteille
     */ 
    public function setSucrage_bouteille($sucrage_bouteille): void {
        $this->sucrage_bouteille = $sucrage_bouteille;
    }

    /**
     * @return $this->type_bouteille
     */ 
    public function getType_bouteille() : string {
        return $this->sucrage_bouteille;
    }

    /**
     * @param $type_bouteille
     */ 
    public function setType_bouteille($type_bouteille): void {
        $this->type_bouteille = $type_bouteille;
    }

    /**
     * @return $this->nouveau
     */ 
    public function getNouveau() : bool {
        return $this->nouveau;
    }

    /**
     * @param $nouveau
     */ 
    public function setNouveau($nouveau): void {
        $this->nouveau=$nouveau;
    }

    /**
     * @return un tableau de toutes les catégories
     */ 
    public static function getAll(): array {
        try {
            if (!isset(self::$_pdo))
                self::initPDO();
            if (!isset(self::$_pdos_selectAll))
                self::initPDOS_selectAll();
            self::$_pdos_selectAll->execute();
            // résultat du fetch dans une instance de CategorieMetier
            $categories = self::$_pdos_selectAll->fetchAll(PDO::FETCH_CLASS,'CategorieMetier');
            return $categories;
        }
        catch (PDOException $e) {
            print($e);
        }
    }

    /**
     * initialisation d'un objet métier à partir d'un enregistrement de catégorie
     * @param $id_categorie un identifiant de catégorie
     * @return l'instance de CategorieMetier associée à $id_categorie
     */ 
    public static function initCategorieMetier($id_categorie) : CategorieMetier {
        try {
	        if (!isset(self::$_pdo))
	            self::initPDO();
	        if (!isset(self::$_pdos_select))
	            self::initPDOS_select();
	        self::$_pdos_select->bindValue(':idB',$id_categorie);
            self::$_pdos_select->execute();

            // résultat du fetch dans une instance de CategorieMetier
	        $lm = self::$_pdos_select->fetchObject('CategorieMetier');
            if (isset($lm) && ! empty($lm))
            {
                $lm->setNouveau(FALSE);
            }

	        if (empty($lm))
                throw new Exception("Catégorie $id_categorie inexistante dans la table Categorie.\n");
	        return $lm;
        }
        catch (PDOException $e) {
	        print($e);
        }
    }

    /**
     * sauvegarde d'un objet métier
     * soit on insère un nouvel objet
     * soit on le met à jour
     */ 
    public function save() : void {
        if (!isset(self::$_pdo))
            self::initPDO();
        if ($this->nouveau) {
            if (!isset(self::$_pdos_insert)) {
	            self::initPDOS_insert();
            }
            self::$_pdos_insert->bindParam(':robeB', $this->robe_bouteille);
            self::$_pdos_insert->bindParam(':sucrageB', $this->sucrage_bouteille);
            self::$_pdos_insert->bindParam(':typeB', $this->type_bouteille);
            self::$_pdos_insert->execute();
            $this->setNouveau(FALSE);
        }
        else {
            if (!isset(self::$_pdos_update))
                self::initPDOS_update();
            self::$_pdos_update->bindParam(':idB', $this->id_categorie);
            self::$_pdos_update->bindParam(':robeB', $this->robe_bouteille);
            self::$_pdos_update->bindParam(':sucrageB', $this->sucrage_bouteille);
            self::$_pdos_update->bindParam(':typeB', $this->type_bouteille);
            self::$_pdos_update->execute();
        }
    }

    /**
     * suppression d'un objet métier
     */ 
    public function delete() :void {
        if (!isset(self::$_pdo))
            self::initPDO();
        if (!$this->nouveau) {
            if (!isset(self::$_pdos_delete)) {
	            self::initPDOS_delete();
            }
            self::$_pdos_delete->bindParam(':idB', $this->id_categorie);
            self::$_pdos_delete->execute();
        }
        $this->setNouveau(TRUE);
    }

    /**
     * nombre d'objets metier disponible dans la table
     */
    public static function getNbCategories() : int {
        if (!isset(self::$_pdos_count)) {
            self::initPDOS_count();
        }
        self::$_pdos_count->execute();
        $resu = self::$_pdos_count->fetch();
        return $resu[0];
    }

    /**
     * affichage élémentaire
     */ 
    public function __toString() : string {
        $ch = "<table border='1'><tr><th>id_categorie</th><th>robe_bouteille</th><th>sucrage_bouteille</th><th>type_bouteille</th><th>nouveau</th></tr><tr>";
        $ch.= "<td>".$this->id_categorie."</td>";
        $ch.= "<td>".$this->robe_bouteille."</td>";
        $ch.= "<td>".$this->sucrage_bouteille."</td>";
        $ch.= "<td>".$this->type_bouteille."</td>";
        $ch.= "<td>".$this->nouveau."</td>";
        $ch.= "</tr></table>";
        return $ch;
    }
}

?>